\name{unsplitSnpSet}
\alias{unsplitSnpSet}
\alias{unsplitSnpSet-methods}
\alias{unsplitSnpSet,list,AnnotatedDataFrame-method}
\title{Combines a list with elements that are AnnotatedSnpSets into one AnnotatedSnpSet.}
\description{
  After splitting a \code{SnpSet} object by chromosome, for instance,
  this function will reproduce the original object from the list.
}
\usage{
unsplitSnpSet(from, annotatedDataFrame, ...)
}
\arguments{
  \item{from}{A \code{list}. Element should be one of the SNP classes
    extending \code{eSet}}
  \item{annotatedDataFrame}{Object of class \code{AnnotatedDataFrame} that
    contains the \code{featureData} prior to splitting.  See the
    example.}
  \item{\dots}{arguments to initialize an instance of the class}
}
\details{
}
\value{
  an object of class \code{class(from[[1]])}

}
\references{}
\author{Robert Scharpf}
\note{
}

\seealso{
  \code{\link[base]{split}}
}

\examples{
  data(sample.snpset)
  x <- split(sample.snpset, chromosome(sample.snpset))
  class(x)
  length(x)

  \dontrun{
    xx <- unsplitSnpSet(x,
                        featureData(sample.snpset),
                        copyNumber=do.call("rbind", lapply(x, copyNumber)),
                        calls=do.call("rbind", lapply(x, calls)),
                        cnConfidence=do.call("rbind", lapply(x, callsConfidence)),
                        callsConfidence=do.call("rbind", lapply(x, callsConfidence)))
    xx
  }
}
\keyword{manip}

